import os
import logging

logger = logging.getLogger("flickrsmartsync")

EXT_IMAGE = ('jpg', 'png', 'jpeg', 'gif', 'bmp')
EXT_VIDEO = ('avi', 'wmv', 'mov', 'mp4', '3gp', 'ogg', 'ogv', 'mts')


class Sync(object):

    def __init__(self, cmd_args, local, remote):
        self.cmd_args = cmd_args
        # Create local and remote objects
        self.local = local
        self.remote = remote

    def start_sync(self):
        # Do the appropriate one time sync
        if self.cmd_args.download:
            self.download()
        elif self.cmd_args.sync_from:
            self.sync()
        else:
            self.upload()
            logger.info('Upload done')
            if self.cmd_args.monitor:
                self.local.watch_for_changes(self.upload)
                self.local.wait_for_quit()

    def sync(self):
        if self.cmd_args.sync_from == "all":
            local_photo_sets = self.local.build_photo_sets(self.cmd_args.sync_path, EXT_IMAGE + EXT_VIDEO)
            remote_photo_sets = self.remote.get_photo_sets()
            # First download complete remote sets that are not local
            for remote_photo_set in remote_photo_sets:
                local_photo_set = os.path.join(self.cmd_args.sync_path, remote_photo_set).replace("/", os.sep)
                if local_photo_set not in local_photo_sets:
                    # TODO: will generate info messages if photo_set is a prefix to other set names
                    self.cmd_args.download = local_photo_set
                    self.download()
            # Now walk our local sets
            for local_photo_set in sorted(local_photo_sets):
                remote_photo_set = local_photo_set.replace(self.cmd_args.sync_path, '').replace("/", os.sep)
                if remote_photo_set not in remote_photo_sets:
                    # doesn't exist remotely, so all files need uploading
                    remote_photos = {}
                else:
                    # filter by what exists remotely, this is a dict of filename->id
                    remote_photos = self.remote.get_photos_in_set(remote_photo_set, get_url=True)
                local_photos = [photo for photo, file_stat in sorted(local_photo_sets[local_photo_set])]
                # download what doesn't exist locally
                for photo in [photo for photo in remote_photos if photo not in local_photos]:
                    self.remote.download(remote_photos[photo], os.path.join(local_photo_set, photo))
                # upload what doesn't exist remotely
                for photo in [photo for photo in local_photos if photo not in remote_photos]:
                    self.remote.upload(os.path.join(local_photo_set, photo), photo, remote_photo_set)          
        else:
            logger.warning("Unsupported sync option: %s" % self.cmd_args.sync_from)

    def download(self):
        # Download to corresponding paths
        for photo_set in self.remote.get_photo_sets():
            if photo_set and (self.cmd_args.download == '.' or photo_set.startswith(self.cmd_args.download)):
                folder = os.path.join(self.cmd_args.sync_path, photo_set)
                logger.info('Getting photos in set [%s]' % photo_set)
                photos = self.remote.get_photos_in_set(photo_set, get_url=True)
                # If Uploaded on unix and downloading on windows & vice versa
                if self.cmd_args.is_windows:
                    folder = folder.replace('/', os.sep)

                for photo in photos:
                    # Adds skips
                    if self.cmd_args.ignore_images and photo.split('.').pop().lower() in EXT_IMAGE:
                        continue
                    elif self.cmd_args.ignore_videos and photo.split('.').pop().lower() in EXT_VIDEO:
                        continue
                    path = os.path.join(folder, photo)
                    if os.path.exists(path):
                        logger.info('Skipped [%s/%s] already downloaded' % (photo_set, photo))
                    else:
                        logger.info('Downloading photo [%s/%s]' % (photo_set, photo))
                        self.remote.download(photos[photo], path)

    def upload(self, specific_path=None):
        if specific_path is None:
            only_dir = self.cmd_args.sync_path
        else:
            only_dir = os.path.dirname(specific_path)
        photo_sets = self.local.build_photo_sets(only_dir, EXT_IMAGE + EXT_VIDEO)
        logger.info('Found %s photo sets' % len(photo_sets))

        if specific_path is None:
            # Show custom set titles
            if self.cmd_args.custom_set:
                for photo_set in photo_sets:
                    logger.info('Set Title: [%s]  Path: [%s]' % (self.remote.get_custom_set_title(photo_set), photo_set))

                if raw_input('Is this your expected custom set titles (y/n):') != 'y':
                    exit(0)

        # Loop through all local photo set map and
        # upload photos that does not exists in online map
        for photo_set in sorted(photo_sets):
            folder = photo_set.replace(self.cmd_args.sync_path, '')
            display_title = self.remote.get_custom_set_title(photo_set)
            logger.info('Getting photos in set [%s]' % display_title)
            photos = self.remote.get_photos_in_set(folder)
            logger.info('Found %s photos' % len(photos))

            for photo, file_stat in sorted(photo_sets[photo_set]):
                # Adds skips
                if self.cmd_args.ignore_images and photo.split('.').pop().lower() in EXT_IMAGE:
                    continue
                elif self.cmd_args.ignore_videos and photo.split('.').pop().lower() in EXT_VIDEO:
                    continue

                if photo in photos or self.cmd_args.is_windows and photo.replace(os.sep, '/') in photos:
                    logger.info('Skipped [%s] already exists in set [%s]' % (photo, display_title))
                else:
                    logger.info('Uploading [%s] to set [%s]' % (photo, display_title))
                    if file_stat.st_size >= 1073741824:
                        logger.error('Skipped [%s] over size limit' % photo)
                        continue
                    file_path = os.path.join(photo_set, photo)                        
                    photo_id = self.remote.upload(file_path, photo, folder)
                    if photo_id:
                        photos[photo] = photo_id


#!/usr/bin/env python
import unittest
import logging
import sys
import os
import time
here = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(here, '..'))
from flickrsmartsync.local import Local

logger = logging.getLogger("flickrsmartsync")
logger.setLevel(logging.WARNING)

class localTest(unittest.TestCase):

    def setUp(self):
        class args:
            sync_path=here+os.sep
            starts_with=None
            keyword=None
        self.local = Local(args())
        self.watch_path = None

    def tearDown(self):
        if self.watch_path:
            os.remove(self.watch_path)

    def test_list_files(self):
        ps = self.local.build_photo_sets(here, ('jpg',))
        self.assertEquals(ps, {here+os.sep+'images': [('new_gradient.jpg', os.stat(here+os.sep+'images'+os.sep+'new_gradient.jpg'))]})

    def test_starts_with(self):
        self.local.cmd_args.starts_with = 'images'
        self.test_list_files()

    def upload_func(self, path):
        self.watch_path=path

    def test_monitor(self):
        self.local.watch_for_changes(self.upload_func)
        p = os.sep.join((here,"images","new.jpg"))
        open(p, "w").write("test")
        time.sleep(1)
        self.assertEquals(self.watch_path, p)


if __name__ == '__main__':
    logging.debug('Started test case')
    unittest.main(verbosity=2)        



#!/usr/bin/env python
import unittest
import logging
import sys
import os

here = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(here, '..'))
import flickrapi
from flickrsmartsync.remote import Remote, KEY, SECRET
logger = logging.getLogger("flickrsmartsync")
logger.setLevel(logging.WARNING)

class authedRemote(Remote):
    def auth_api(self):
        token = "72157645636119732-71cd968268ffd2e5"
        self.api = flickrapi.FlickrAPI(KEY, SECRET, username="rqpmbdqm24", token=token)
        return token

class remoteTest(unittest.TestCase):

    def setUp(self):
        class args:
            username=None
            sync_path=here+os.sep
            custom_set=None
            update_custom_set=None
            custom_set_builder=None
            is_windows=False
            download=None
        self.remote = authedRemote(args())
        self.deletePhotoset = None
        self.deleteFile = None
        self.deletePhoto = None

    def tearDown(self):
        if self.deletePhotoset:
            try:
                self.remote.api.photosets_delete(photoset_id=self.deletePhotoset)
            except:
                pass
        if self.deletePhoto:
            try:
                self.remote.api.photos_delete(photo_id=self.deletePhoto)
            except:
                pass
        if self.deleteFile:
            os.remove(self.deleteFile)

    def test_list_sets(self):
        self.assertEquals(self.remote.get_photo_sets(), {'Gradient test images': u'72157645524834044'})

    def test_list_photos(self):
        self.assertEquals(self.remote.get_photos_in_set('Gradient test images'),{'Black gradient (reasonable size).jpg': u'14491535316'})

    def test_add_to_set(self):
        self.remote.add_to_photo_set(14524802935, 'Stuff2')
        self.deletePhotoset = self.remote.get_photo_sets()['Stuff2']
        self.assertEquals(len(self.remote.get_photos_in_set('Stuff2')),1)
        self.remote.add_to_photo_set(14491535316, 'Stuff2')
        self.assertEquals(len(self.remote.get_photos_in_set('Stuff2')),2)

    def test_addPhoto(self):
        self.deletePhoto = self.remote.upload(here+'/images/new_gradient.jpg', 'new_gradient.jpg', 'images')
        self.deletePhotoset = self.remote.get_photo_sets()['images']
        self.assertEquals(len(self.remote.get_photos_in_set('images')),1)

    def test_customSet(self):
        self.remote.cmd_args.custom_set = ".*im(.*)"
        self.deletePhoto = self.remote.upload(here+'/images/new_gradient.jpg', 'new_gradient.jpg', 'images')
        self.deletePhotoset = self.remote.get_photo_sets()['images']
        self.assertEquals(self.remote.api.photosets_getInfo(photoset_id=self.deletePhotoset).find("photoset").find("title").text, "ages")
        self.remote.cmd_args.update_custom_set = True
        self.remote.cmd_args.custom_set = ".*i(.*)"
        self.remote.update_photo_sets_map()
        self.assertEquals(self.remote.api.photosets_getInfo(photoset_id=self.deletePhotoset).find("photoset").find("title").text, "mages")

    def test_download(self):
        images = self.remote.get_photos_in_set('Gradient test images', get_url=True)
        self.assertEquals(len(images), 1)
        self.deleteFile = here+os.sep+"tmp.jpg"
        self.remote.download(images.values()[0], self.deleteFile)
        self.assertEquals(os.path.isfile(self.deleteFile), 1)

if __name__ == '__main__':
    unittest.main(verbosity=2)



from iptcinfo import IPTCInfo
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import time
import logging
import os

logger = logging.getLogger("flickrsmartsync")


class Local(object):
    def __init__(self, cmd_args):
        self.cmd_args = cmd_args

    def build_photo_sets(self, path, extensions):
        # Build your local photo sets
        photo_sets = {}
        skips_root = []
        keywords = set(self.cmd_args.keyword) if self.cmd_args.keyword else ()

        for r, dirs, files in os.walk(path):

            if self.cmd_args.starts_with and not r.startswith('{}{}'.format(self.cmd_args.sync_path, self.cmd_args.starts_with)):
                continue

            files = [f for f in files if not f.startswith('.')]
            dirs[:] = [d for d in dirs if not d.startswith('.')]

            for file in files:
                if not file.startswith('.'):
                    ext = file.lower().split('.').pop()
                    if ext in extensions:
                        if r == self.cmd_args.sync_path:
                            skips_root.append(file)
                        else:
                            # filter by keywords
                            if keywords:
                                file_path = os.path.join(r, file)
                                info = IPTCInfo(file_path, force=True)
                                matches = keywords.intersection(info.keywords)
                                if not matches:
                                    # no matching keyword(s) found, skip file
                                    logger.info('Skipped [%s] does not match any keyword %s' % (file, list(keywords)))
                                    continue
                        
                            photo_sets.setdefault(r, [])
                            file_path = os.path.join(r, file)
                            file_stat = os.stat(file_path)
                            photo_sets[r].append((file, file_stat))

        if skips_root:
            logger.warn('To avoid disorganization on flickr sets root photos are not synced, skipped these photos: %s' % skips_root)
            logger.warn('Try to sync at top most level of your photos directory')
        return photo_sets

    def watch_for_changes(self, upload_func):
        logger.info('Monitoring [{}]'.format(self.cmd_args.sync_path))
        event_handler = WatchEventHandler(self.cmd_args.sync_path, upload_func)
        self.observer = Observer()
        self.observer.schedule(event_handler, self.cmd_args.sync_path, recursive=True)
        self.observer.start()

    def wait_for_quit(self):
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.observer.stop()
        self.observer.join()


class WatchEventHandler(FileSystemEventHandler):
    sync_path = None

    def __init__(self, sync_path, upload_func):
        self.sync_path = sync_path.rstrip(os.sep)
        self.upload_func = upload_func

    def on_created(self, event):
        super(WatchEventHandler, self).on_created(event)

        if not event.is_directory:
            self.upload_func(event.src_path)

    def on_moved(self, event):
        super(WatchEventHandler, self).on_moved(event)

        if not event.is_directory and os.path.dirname(event.dest_path).replace(self.sync_path, ''):
            self.upload_func(event.dest_path)        



#!/usr/bin/env python
import unittest
import logging
import sys
import os
import copy

import time
here = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(here, '..'))
from flickrsmartsync.sync import Sync
logger = logging.getLogger("flickrsmartsync")
logger.setLevel(logging.WARNING)

fakestat = os.stat(__file__)
fakeid = 45
class fakeLocal:
    files = {}
    def __init__(self):
        self.files.clear()
        self.files.update({here + os.sep + "dirname": [("file1.jpg", fakestat), ("file2.avi", fakestat)]})
    
    def build_photo_sets(self, specific_path, exts):
        return self.files
    
class fakeRemote:
    def __init__(self):
        self.photo_sets_map = {"dirname": "12345"}
        self.files = {"12345": {"file3.jpg": 23, "file4.avi": 23}}  
    def get_photo_sets(self):
        return self.photo_sets_map
    def get_custom_set_title(self, path):
        return path.split('/').pop()      
    def get_photos_in_set(self, folder, get_url=False):
        return self.files[self.photo_sets_map[folder]]  
    def upload(self, file_path, photo, folder):
        self.files[self.photo_sets_map[folder]][photo] = fakeid
    def download(self, url, path):
        fakeLocal.files.values()[0].append((os.path.basename(path), fakestat))

class syncTest(unittest.TestCase):
    maxDiff=None

    def setUp(self):
        class args:
            sync_path=here+os.sep
            custom_set=None
            ignore_images=False
            ignore_videos=False
            is_windows=False
            download="."
            sync_from="all"
        self.local = fakeLocal()
        self.remote = fakeRemote()
        self.sync = Sync(args(), self.local, self.remote)

    def tearDown(self):
        pass

    def test_upload(self):
        expected = fakeRemote().files
        for f, s in self.local.files.values()[0]:
            expected.values()[0][f] = fakeid
        self.sync.upload()
        self.assertEquals(self.remote.files, expected)

    def test_download(self):
        expected = copy.deepcopy(fakeLocal().files)
        expected.values()[0] += [(x, fakestat) for x in self.remote.files.values()[0]]
        self.sync.download()
        self.assertEquals(self.local.files, expected)

    def test_sync(self):
        expectedr = fakeRemote().files
        for f, s in self.local.files.values()[0]:
            expectedr.values()[0][f] = fakeid        
        expectedl = copy.deepcopy(fakeLocal().files)
        expectedl.values()[0] += [(x, fakestat) for x in self.remote.files.values()[0]]
        self.sync.sync()
        self.assertEquals(self.remote.files, expectedr)
        self.assertEquals(self.local.files, expectedl)
        
if __name__ == '__main__':
    logging.debug('Started test case')
    unittest.main(verbosity=2)      



#
# -*- coding: utf-8 -*-
import argparse
import os
import logging
from logging.handlers import SysLogHandler
from sync import Sync
from local import Local
from remote import Remote

__author__ = 'faisal'

logging.basicConfig()
logger = logging.getLogger("flickrsmartsync")
hdlr = SysLogHandler()
formatter = logging.Formatter('flickrsmartsync %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr)
logger.setLevel(logging.INFO)

def main():
    parser = argparse.ArgumentParser(description='Sync current folder to your flickr account.')
    parser.add_argument('--monitor', action='store_true', help='starts a daemon after sync for monitoring')
    parser.add_argument('--starts-with', type=str, help='only sync that path that starts with')
    parser.add_argument('--download', type=str, help='download the photos from flickr specify a path or . for all')
    parser.add_argument('--ignore-videos', action='store_true', help='ignore video files')
    parser.add_argument('--ignore-images', action='store_true', help='ignore image files')
    parser.add_argument('--version', action='store_true', help='output current version')
    parser.add_argument('--sync-path', type=str, default=os.getcwd(),
                        help='specify the sync folder (default is current dir)')
    parser.add_argument('--sync-from', type=str, help='If set to "all", then upload anything that isn\'t on flickr, and download anything that isn\'t on the local filesystem')
    parser.add_argument('--custom-set', type=str, help='customize your set name from path with regex')
    parser.add_argument('--custom-set-builder', type=str, help='build your custom set title (default just merge groups)')
    parser.add_argument('--update-custom-set', action='store_true', help='updates your set title from custom set')
    parser.add_argument('--username', type=str, help='token username') #token username argument for api
    parser.add_argument('--keyword', action='append', type=str, help='only upload files matching this keyword')

    args = parser.parse_args()

    if args.version:
        # todo get from setup.cfg
        logger.info('0.2.00')
        exit()

    # validate args
    args.is_windows = os.name == 'nt'
    args.sync_path = args.sync_path.rstrip(os.sep) + os.sep
    if not os.path.exists(args.sync_path):
        logger.error('Sync path does not exists')
        exit(0)

    local = Local(args)
    remote = Remote(args)
    sync = Sync(args, local, remote)
    sync.start_sync()




import HTMLParser
import json
import os
import re
import urllib
import flickrapi
import logging

logger = logging.getLogger("flickrsmartsync")

#  flickr api keys
KEY = 'f7da21662566bc773c7c750ddf7030f7'
SECRET = 'c329cdaf44c6d3f3'

# number of retries for downloads
RETRIES = 5


class Remote(object):

    def __init__(self, cmd_args):
        # Command line arguments
        self.cmd_args = cmd_args
        token = self.auth_api()

        # Common arguments
        self.args = {'format': 'json', 'nojsoncallback': 1, 'auth_token': token}

        # photo_sets_map[folder] = id
        self.update_photo_sets_map()

    def auth_api(self):
        self.api = flickrapi.FlickrAPI(KEY, SECRET, username=self.cmd_args.username)  # pass username argument to api
        # api.token.path = 'flickr.token.txt'

        # Ask for permission
        (token, frob) = self.api.get_token_part_one(perms='delete')
        if not token:
            raw_input("Please authorized this app then hit enter:")
        try:
            token = self.api.get_token_part_two((token, frob))
        except:
            logger.error('Please authorized to use')
            exit(0)
        return token

    # custom set builder
    def get_custom_set_title(self, path):
        title = path.split('/').pop()

        if self.cmd_args.custom_set:
            m = re.match(self.cmd_args.custom_set, path)
            if m:
                if not self.cmd_args.custom_set_builder:
                    title = '-'.join(m.groups())
                elif m.groupdict():
                    title = self.cmd_args.custom_set_builder.format(**m.groupdict())
                else:
                    title = self.cmd_args.custom_set_builder.format(*m.groups())
        return title

    # For adding photo to set
    def add_to_photo_set(self, photo_id, folder):
        # If photoset not found in online map create it else add photo to it
        # Always upload unix style
        if self.cmd_args.is_windows:
            folder = folder.replace(os.sep, '/')

        if folder not in self.photo_sets_map:
            photosets_args = self.args.copy()
            custom_title = self.get_custom_set_title(self.cmd_args.sync_path + folder)
            photosets_args.update({'primary_photo_id': photo_id,
                                   'title': custom_title,
                                   'description': folder})
            photo_set = json.loads(self.api.photosets_create(**photosets_args))
            self.photo_sets_map[folder] = photo_set['photoset']['id']
            logger.info('Created set [%s] and added photo' % custom_title)
        else:
            photosets_args = self.args.copy()
            photosets_args.update({'photoset_id': self.photo_sets_map.get(folder), 'photo_id': photo_id})
            result = json.loads(self.api.photosets_addPhoto(**photosets_args))
            if result.get('stat') == 'ok':
                logger.info('Successfully added photo to %s' % folder)
            else:
                logger.error(result)

    # Get photos in a set
    def get_photos_in_set(self, folder, get_url=False):
        # bug on non utf8 machines dups
        folder = folder.encode('utf-8') if isinstance(folder, unicode) else folder

        photos = {}
        # Always upload unix style
        if self.cmd_args.is_windows:
            folder = folder.replace(os.sep, '/')

        if folder in self.photo_sets_map:
            photoset_args = self.args.copy()
            page = 1
            while True:
                photoset_args.update({'photoset_id': self.photo_sets_map[folder], 'page': page})
                if get_url:
                    photoset_args['extras'] = 'url_o,media'
                page += 1
                photos_in_set = json.loads(self.api.photosets_getPhotos(**photoset_args))
                if photos_in_set['stat'] != 'ok':
                    break

                for photo in photos_in_set['photoset']['photo']:
                    title = photo['title'].encode('utf-8')
                    # add missing extension if not present (take a guess as api original_format argument not working)
                    split = title.split(".")
                    # assume valid file extension is less than or equal to 5 characters and not all digits
                    if len(split) < 2 or len(split[-1]) > 5 or split[-1].isdigit():
                        if photo.get('media') == 'video':
                            title += ".mp4"
                        else:
                            title += ".jpg"
                    if get_url and photo.get('media') == 'video':
                        photo_args = self.args.copy()
                        photo_args['photo_id'] = photo['id']
                        sizes = json.loads(self.api.photos_getSizes(**photo_args))
                        if sizes['stat'] != 'ok':
                            continue

                        original = filter(lambda s: s['label'].startswith('Video Original') and s['media'] == 'video', sizes['sizes']['size'])
                        if original:
                            photos[title] = original.pop()['source']
                            
                    else:
                        photos[title] = photo['url_o'] if get_url else photo['id']

        return photos

    def get_photo_sets(self):
        return self.photo_sets_map

    def update_photo_sets_map(self):
        # Get your photosets online and map it to your local
        html_parser = HTMLParser.HTMLParser()
        photosets_args = self.args.copy()
        page = 1
        self.photo_sets_map = {}

        while True:
            logger.info('Getting photosets page %s' % page)
            photosets_args.update({'page': page, 'per_page': 500})
            sets = json.loads(self.api.photosets_getList(**photosets_args))
            page += 1
            if not sets['photosets']['photoset']:
                break

            for current_set in sets['photosets']['photoset']:
                # Make sure it's the one from backup format
                desc = html_parser.unescape(current_set['description']['_content'])
                desc = desc.encode('utf-8') if isinstance(desc, unicode) else desc
                if desc:
                    self.photo_sets_map[desc] = current_set['id']
                    title = self.get_custom_set_title(self.cmd_args.sync_path + desc)
                    if self.cmd_args.update_custom_set and title != current_set['title']['_content']:
                        update_args = self.args.copy()
                        update_args.update({
                            'photoset_id': current_set['id'],
                            'title': title,
                            'description': desc
                        })
                        logger.info('Updating custom title [%s]...' % title)
                        json.loads(self.api.photosets_editMeta(**update_args))
                        logger.info('done')

    def upload(self, file_path, photo, folder):
        upload_args = {
            'auth_token': self.args["auth_token"],
            # (Optional) The title of the photo.
            'title': photo,
            # (Optional) A description of the photo. May contain some limited HTML.
            'description': folder,
            # (Optional) Set to 0 for no, 1 for yes. Specifies who can view the photo.
            'is_public': 0,
            'is_friend': 0,
            'is_family': 0,
            # (Optional) Set to 1 for Safe, 2 for Moderate, or 3 for Restricted.
            'safety_level': 1,
            # (Optional) Set to 1 for Photo, 2 for Screenshot, or 3 for Other.
            'content_type': 1,
            # (Optional) Set to 1 to keep the photo in global search results, 2 to hide from public searches.
            'hidden': 2
        }

        for i in range(RETRIES):
            try:
                upload = self.api.upload(file_path, None, **upload_args)
                photo_id = upload.find('photoid').text
                self.add_to_photo_set(photo_id, folder)
                return photo_id
            except Exception as e:
                logger.warning("Retrying upload of %s/%s after error: %s" % (folder, photo, e))
        logger.error("Failed upload of %s/%s after %d retries" % (folder, photo, RETRIES))

    def download(self, url, path):
        folder = os.path.dirname(path)
        if not os.path.isdir(folder):
            os.makedirs(folder)   
        for i in range(RETRIES):
            try:
                return urllib.urlretrieve(url, path)
            except Exception as e:
                logger.warning("Retrying download of %s after error: %s" % (path, e))
        # failed many times
        logger.error("Failed to download %s after %d retries" % (path, RETRIES))
